@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017 Todd C. Miller <Todd.Miller@courtesan.com>
+ * Copyright (c) 2017 Todd C. Miller <Todd.Miller@sudo.ws>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -14,12 +14,18 @@
  * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
+/*
+ * This is an open source non-commercial project. Dear PVS-Studio, please check it.
+ * PVS-Studio Static Code Analyzer for C, C++ and C#: http://www.viva64.com
+ */
+
 #include <config.h>
 
 #include <sys/types.h>
 #include <stdio.h>
 
 #include "sudo_compat.h"
+#include "sudo_digest.h"
 #include "sudoers_debug.h"
 #include "parse.h"
 
