@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2010-2017 Todd C. Miller <Todd.Miller@courtesan.com>
+ * Copyright (c) 2010-2017 Todd C. Miller <Todd.Miller@sudo.ws>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -14,6 +14,11 @@
  * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
+/*
+ * This is an open source non-commercial project. Dear PVS-Studio, please check it.
+ * PVS-Studio Static Code Analyzer for C, C++ and C#: http://www.viva64.com
+ */
+
 #include <config.h>
 
 #include <sys/types.h>
@@ -35,7 +40,6 @@
 #include "sudoers.h"
 #include "sudoers_version.h"
 #include "interfaces.h"
-#include "parse.h" /* for parse_timeout() */
 
 /*
  * Info passed in from the sudo front-end.
@@ -68,6 +72,25 @@ extern __dso_public struct policy_plugin sudoers_policy;
 extern char *login_style;
 #endif /* HAVE_BSD_AUTH_H */
 
+static int
+parse_bool(const char *line, int varlen, int *flags, int fval)
+{
+    debug_decl(parse_bool, SUDOERS_DEBUG_PLUGIN)
+
+    switch (sudo_strtobool(line + varlen + 1)) {
+    case true:
+	SET(*flags, fval);
+	debug_return_int(true);
+    case false:
+	CLR(*flags, fval);
+	debug_return_int(false);
+    default:
+	sudo_warn(U_("invalid %.*s set by sudo front-end"),
+	    varlen, line);
+	debug_return_int(-1);
+    }
+}
+
 /*
  * Deserialize args, settings and user_info arrays.
  * Fills in struct sudo_user and other common sudoers state.
@@ -79,15 +102,26 @@ sudoers_policy_deserialize_info(void *v, char **runas_user, char **runas_group)
     char * const *cur;
     const char *p, *errstr, *groups = NULL;
     const char *remhost = NULL;
+    bool uid_set = false, gid_set = false;
     int flags = 0;
     debug_decl(sudoers_policy_deserialize_info, SUDOERS_DEBUG_PLUGIN)
 
-#define MATCHES(s, v) (strncmp(s, v, sizeof(v) - 1) == 0)
+#define MATCHES(s, v)	\
+    (strncmp((s), (v), sizeof(v) - 1) == 0)
+
+#define CHECK(s, v) do {	\
+    if ((s)[sizeof(v) - 1] == '\0') { \
+	sudo_warn(U_("invalid %.*s set by sudo front-end"), \
+	    (int)(sizeof(v) - 2), v); \
+	goto bad; \
+    } \
+} while (0)
 
     /* Parse sudo.conf plugin args. */
     if (info->plugin_args != NULL) {
 	for (cur = info->plugin_args; *cur != NULL; cur++) {
 	    if (MATCHES(*cur, "sudoers_file=")) {
+		CHECK(*cur, "sudoers_file=");
 		sudoers_file = *cur + sizeof("sudoers_file=") - 1;
 		continue;
 	    }
@@ -119,10 +153,12 @@ sudoers_policy_deserialize_info(void *v, char **runas_user, char **runas_group)
 		continue;
 	    }
 	    if (MATCHES(*cur, "ldap_conf=")) {
+		CHECK(*cur, "ldap_conf=");
 		path_ldap_conf = *cur + sizeof("ldap_conf=") - 1;
 		continue;
 	    }
 	    if (MATCHES(*cur, "ldap_secret=")) {
+		CHECK(*cur, "ldap_secret=");
 		path_ldap_secret = *cur + sizeof("ldap_secret=") - 1;
 		continue;
 	    }
@@ -143,94 +179,110 @@ sudoers_policy_deserialize_info(void *v, char **runas_user, char **runas_group)
 	    continue;
 	}
 	if (MATCHES(*cur, "runas_user=")) {
+	    CHECK(*cur, "runas_user=");
 	    *runas_user = *cur + sizeof("runas_user=") - 1;
 	    sudo_user.flags |= RUNAS_USER_SPECIFIED;
 	    continue;
 	}
 	if (MATCHES(*cur, "runas_group=")) {
+	    CHECK(*cur, "runas_group=");
 	    *runas_group = *cur + sizeof("runas_group=") - 1;
 	    sudo_user.flags |= RUNAS_GROUP_SPECIFIED;
 	    continue;
 	}
 	if (MATCHES(*cur, "prompt=")) {
+	    /* Allow epmpty prompt. */
 	    user_prompt = *cur + sizeof("prompt=") - 1;
 	    def_passprompt_override = true;
 	    continue;
 	}
 	if (MATCHES(*cur, "set_home=")) {
-	    if (sudo_strtobool(*cur + sizeof("set_home=") - 1) == true)
-		SET(flags, MODE_RESET_HOME);
+	    if (parse_bool(*cur, sizeof("set_home") - 1, &flags,
+		MODE_RESET_HOME) == -1)
+		goto bad;
 	    continue;
 	}
 	if (MATCHES(*cur, "preserve_environment=")) {
-	    if (sudo_strtobool(*cur + sizeof("preserve_environment=") - 1) == true)
-		SET(flags, MODE_PRESERVE_ENV);
+	    if (parse_bool(*cur, sizeof("preserve_environment") - 1, &flags,
+		MODE_PRESERVE_ENV) == -1)
+		goto bad;
 	    continue;
 	}
 	if (MATCHES(*cur, "run_shell=")) {
-	    if (sudo_strtobool(*cur + sizeof("run_shell=") - 1) == true)
-		SET(flags, MODE_SHELL);
+	    if (parse_bool(*cur, sizeof("run_shell") -1, &flags,
+		MODE_SHELL) == -1)
+		goto bad;
 	    continue;
 	}
 	if (MATCHES(*cur, "login_shell=")) {
-	    if (sudo_strtobool(*cur + sizeof("login_shell=") - 1) == true) {
-		SET(flags, MODE_LOGIN_SHELL);
-		def_env_reset = true;
-	    }
+	    if (parse_bool(*cur, sizeof("login_shell") - 1, &flags,
+		MODE_LOGIN_SHELL) == -1)
+		goto bad;
 	    continue;
 	}
 	if (MATCHES(*cur, "implied_shell=")) {
-	    if (sudo_strtobool(*cur + sizeof("implied_shell=") - 1) == true)
-		SET(flags, MODE_IMPLIED_SHELL);
+	    if (parse_bool(*cur, sizeof("implied_shell") - 1, &flags,
+		MODE_IMPLIED_SHELL) == -1)
+		goto bad;
 	    continue;
 	}
 	if (MATCHES(*cur, "preserve_groups=")) {
-	    if (sudo_strtobool(*cur + sizeof("preserve_groups=") - 1) == true)
-		SET(flags, MODE_PRESERVE_GROUPS);
+	    if (parse_bool(*cur, sizeof("preserve_groups") - 1, &flags,
+		MODE_PRESERVE_GROUPS) == -1)
+		goto bad;
 	    continue;
 	}
 	if (MATCHES(*cur, "ignore_ticket=")) {
-	    if (sudo_strtobool(*cur + sizeof("ignore_ticket=") - 1) == true)
-		SET(flags, MODE_IGNORE_TICKET);
+	    if (parse_bool(*cur, sizeof("ignore_ticket") -1, &flags,
+		MODE_IGNORE_TICKET) == -1)
+		goto bad;
 	    continue;
 	}
 	if (MATCHES(*cur, "noninteractive=")) {
-	    if (sudo_strtobool(*cur + sizeof("noninteractive=") - 1) == true)
-		SET(flags, MODE_NONINTERACTIVE);
+	    if (parse_bool(*cur, sizeof("noninteractive") - 1, &flags,
+		MODE_NONINTERACTIVE) == -1)
+		goto bad;
 	    continue;
 	}
 	if (MATCHES(*cur, "sudoedit=")) {
-	    if (sudo_strtobool(*cur + sizeof("sudoedit=") - 1) == true)
-		SET(flags, MODE_EDIT);
+	    if (parse_bool(*cur, sizeof("sudoedit") - 1, &flags,
+		MODE_EDIT) == -1)
+		goto bad;
 	    continue;
 	}
 	if (MATCHES(*cur, "login_class=")) {
+	    CHECK(*cur, "login_class=");
 	    login_class = *cur + sizeof("login_class=") - 1;
 	    def_use_loginclass = true;
 	    continue;
 	}
 #ifdef HAVE_PRIV_SET
 	if (MATCHES(*cur, "runas_privs=")) {
+	    CHECK(*cur, "runas_privs=");
 	    def_privs = *cur + sizeof("runas_privs=") - 1;
 	    continue;
 	}
 	if (MATCHES(*cur, "runas_limitprivs=")) {
+	    CHECK(*cur, "runas_limitprivs=");
 	    def_limitprivs = *cur + sizeof("runas_limitprivs=") - 1;
 	    continue;
 	}
 #endif /* HAVE_PRIV_SET */
 #ifdef HAVE_SELINUX
 	if (MATCHES(*cur, "selinux_role=")) {
+	    CHECK(*cur, "selinux_role=");
 	    user_role = *cur + sizeof("selinux_role=") - 1;
 	    continue;
 	}
 	if (MATCHES(*cur, "selinux_type=")) {
+	    CHECK(*cur, "selinux_type=");
 	    user_type = *cur + sizeof("selinux_type=") - 1;
 	    continue;
 	}
 #endif /* HAVE_SELINUX */
 #ifdef HAVE_BSD_AUTH_H
 	if (MATCHES(*cur, "bsdauth_type=")) {
+	    CHECK(*cur, "login_style=");
 	    login_style = *cur + sizeof("bsdauth_type=") - 1;
 	    continue;
 	}
@@ -254,6 +306,7 @@ sudoers_policy_deserialize_info(void *v, char **runas_user, char **runas_group)
 	    continue;
 	}
 	if (MATCHES(*cur, "remote_host=")) {
+	    CHECK(*cur, "remote_host=");
 	    remhost = *cur + sizeof("remote_host=") - 1;
 	    continue;
 	}
@@ -271,6 +324,7 @@ sudoers_policy_deserialize_info(void *v, char **runas_user, char **runas_group)
 	}
 #ifdef ENABLE_SUDO_PLUGIN_API
 	if (MATCHES(*cur, "plugin_dir=")) {
+	    CHECK(*cur, "plugin_dir=");
 	    path_plugin_dir = *cur + sizeof("plugin_dir=") - 1;
 	    continue;
 	}
@@ -280,6 +334,7 @@ sudoers_policy_deserialize_info(void *v, char **runas_user, char **runas_group)
     user_umask = (mode_t)-1;
     for (cur = info->user_info; *cur != NULL; cur++) {
 	if (MATCHES(*cur, "user=")) {
+	    CHECK(*cur, "user=");
 	    if ((user_name = strdup(*cur + sizeof("user=") - 1)) == NULL)
 		goto oom;
 	    continue;
@@ -291,6 +346,7 @@ sudoers_policy_deserialize_info(void *v, char **runas_user, char **runas_group)
 		sudo_warnx(U_("%s: %s"), *cur, U_(errstr));
 		goto bad;
 	    }
+	    uid_set = true;
 	    continue;
 	}
 	if (MATCHES(*cur, "gid=")) {
@@ -300,18 +356,22 @@ sudoers_policy_deserialize_info(void *v, char **runas_user, char **runas_group)
 		sudo_warnx(U_("%s: %s"), *cur, U_(errstr));
 		goto bad;
 	    }
+	    gid_set = true;
 	    continue;
 	}
 	if (MATCHES(*cur, "groups=")) {
+	    CHECK(*cur, "groups=");
 	    groups = *cur + sizeof("groups=") - 1;
 	    continue;
 	}
 	if (MATCHES(*cur, "cwd=")) {
+	    CHECK(*cur, "cwd=");
 	    if ((user_cwd = strdup(*cur + sizeof("cwd=") - 1)) == NULL)
 		goto oom;
 	    continue;
 	}
 	if (MATCHES(*cur, "tty=")) {
+	    CHECK(*cur, "tty=");
 	    if ((user_ttypath = strdup(*cur + sizeof("tty=") - 1)) == NULL)
 		goto oom;
 	    user_tty = user_ttypath;
@@ -320,6 +380,7 @@ sudoers_policy_deserialize_info(void *v, char **runas_user, char **runas_group)
 	    continue;
 	}
 	if (MATCHES(*cur, "host=")) {
+	    CHECK(*cur, "host=");
 	    if ((user_host = strdup(*cur + sizeof("host=") - 1)) == NULL)
 		goto oom;
 	    if ((p = strchr(user_host, '.')) != NULL) {
@@ -353,7 +414,7 @@ sudoers_policy_deserialize_info(void *v, char **runas_user, char **runas_group)
 	}
 	if (MATCHES(*cur, "sid=")) {
 	    p = *cur + sizeof("sid=") - 1;
-	    sudo_user.sid = (pid_t) sudo_strtoid(p, NULL, NULL, &errstr);
+	    user_sid = (pid_t) sudo_strtoid(p, NULL, NULL, &errstr);
 	    if (errstr != NULL) {
 		sudo_warnx(U_("%s: %s"), *cur, U_(errstr));
 		goto bad;
@@ -370,6 +431,25 @@ sudoers_policy_deserialize_info(void *v, char **runas_user, char **runas_group)
 	    continue;
 	}
     }
+
+    /* User name, user ID, group ID and host name must be specified. */
+    if (user_name == NULL) {
+	sudo_warnx(U_("user name not set by sudo front-end"));
+	goto bad;
+    }
+    if (!uid_set) {
+	sudo_warnx(U_("user ID not set by sudo front-end"));
+	goto bad;
+    }
+    if (!gid_set) {
+	sudo_warnx(U_("group ID not set by sudo front-end"));
+	goto bad;
+    }
+    if (user_host == NULL) {
+	sudo_warnx(U_("host name not set by sudo front-end"));
+	goto bad;
+    }
+
     if ((user_runhost = strdup(remhost ? remhost : user_host)) == NULL)
 	goto oom;
     if ((p = strchr(user_runhost, '.')) != NULL) {
@@ -389,7 +469,7 @@ sudoers_policy_deserialize_info(void *v, char **runas_user, char **runas_group)
 	/* user_ttypath remains NULL */
     }
 
-    if (groups != NULL && groups[0] != '\0') {
+    if (groups != NULL) {
 	/* sudo_parse_gids() will print a warning on error. */
 	user_ngids = sudo_parse_gids(groups, &user_gid, &user_gids);
 	if (user_ngids == -1)
@@ -402,6 +482,10 @@ sudoers_policy_deserialize_info(void *v, char **runas_user, char **runas_group)
 	umask(user_umask);
     }
 
+    /* Always reset the environment for a login shell. */
+    if (ISSET(flags, MODE_LOGIN_SHELL))
+	def_env_reset = true;
+
     /* Some systems support fexecve() which we use for digest matches. */
     cmnd_fd = -1;
 
@@ -517,7 +601,10 @@ sudoers_policy_exec_setup(char *argv[], char *envp[], mode_t cmnd_umask,
 	gid_t egid;
 	size_t glsize;
 	char *cp, *gid_list;
-	struct gid_list *gidlist = sudo_get_gidlist(runas_pw);
+	struct gid_list *gidlist;
+
+	/* Only use results from a group db query, not the front end. */
+	gidlist = sudo_get_gidlist(runas_pw, ENTRY_TYPE_QUERIED);
 
 	/* We reserve an extra spot in the list for the effective gid. */
 	glsize = sizeof("runas_groups=") - 1 +
@@ -531,7 +618,7 @@ sudoers_policy_exec_setup(char *argv[], char *envp[], mode_t cmnd_umask,
 	/* On BSD systems the effective gid is the first group in the list. */
 	egid = runas_gr ? (unsigned int)runas_gr->gr_gid :
 	    (unsigned int)runas_pw->pw_gid;
-	len = snprintf(cp, glsize - (cp - gid_list), "%u", egid);
+	len = snprintf(cp, glsize - (cp - gid_list), "%u", (unsigned int)egid);
 	if (len < 0 || (size_t)len >= glsize - (cp - gid_list)) {
 	    sudo_warnx(U_("internal error, %s overflow"), __func__);
 	    free(gid_list);
@@ -594,9 +681,9 @@ sudoers_policy_exec_setup(char *argv[], char *envp[], mode_t cmnd_umask,
 	    goto oom;
     }
     if (def_command_timeout > 0 || user_timeout > 0) {
-	int timeout = def_command_timeout;
-	if (timeout <= 0 || user_timeout < timeout)
-	    timeout = user_timeout;
+	int timeout = user_timeout;
+	if (timeout == 0 || def_command_timeout < timeout)
+	    timeout = def_command_timeout;
 	if (asprintf(&command_info[info_len++], "timeout=%u", timeout) == -1)
 	    goto oom;
     }
@@ -641,7 +728,12 @@ sudoers_policy_exec_setup(char *argv[], char *envp[], mode_t cmnd_umask,
     }
 #endif /* HAVE_SELINUX */
 
-    /* Fill in exec environment info */
+    /* Free on exit; they are not available in the close function. */
+    sudoers_gc_add(GC_VECTOR, argv);
+    sudoers_gc_add(GC_VECTOR, envp);
+    sudoers_gc_add(GC_VECTOR, command_info);
+
+    /* Fill in exec environment info. */
     *(exec_args->argv) = argv;
     *(exec_args->envp) = envp;
     *(exec_args->info) = command_info;
@@ -771,7 +863,7 @@ sudoers_policy_check(int argc, char * const argv[], char *env_add[],
     exec_args.envp = user_env_out;
     exec_args.info = command_infop;
 
-    ret = sudoers_policy_main(argc, argv, 0, env_add, &exec_args);
+    ret = sudoers_policy_main(argc, argv, 0, env_add, false, &exec_args);
     if (ret == true && sudo_version >= SUDO_API_MKVERSION(1, 3)) {
 	/* Unset close function if we don't need it to avoid extra process. */
 	if (!def_log_input && !def_log_output && !def_use_pty &&
@@ -789,7 +881,7 @@ sudoers_policy_validate(void)
     user_cmnd = "validate";
     SET(sudo_mode, MODE_VALIDATE);
 
-    debug_return_int(sudoers_policy_main(0, NULL, I_VERIFYPW, NULL, NULL));
+    debug_return_int(sudoers_policy_main(0, NULL, I_VERIFYPW, NULL, false, NULL));
 }
 
 static void
@@ -817,8 +909,6 @@ sudoers_policy_list(int argc, char * const argv[], int verbose,
 	SET(sudo_mode, MODE_CHECK);
     else
 	SET(sudo_mode, MODE_LIST);
-    if (verbose)
-	long_list = 1;
     if (list_user) {
 	list_pw = sudo_getpwnam(list_user);
 	if (list_pw == NULL) {
@@ -826,7 +916,7 @@ sudoers_policy_list(int argc, char * const argv[], int verbose,
 	    debug_return_int(-1);
 	}
     }
-    ret = sudoers_policy_main(argc, argv, I_LISTPW, NULL, NULL);
+    ret = sudoers_policy_main(argc, argv, I_LISTPW, NULL, verbose, NULL);
     if (list_user) {
 	sudo_pw_delref(list_pw);
 	list_pw = NULL;
