@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2010, 2012 Todd C. Miller <Todd.Miller@courtesan.com>
+ * Copyright (c) 2010-2014 Todd C. Miller <Todd.Miller@sudo.ws>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -14,43 +14,30 @@
  * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
+/*
+ * This is an open source non-commercial project. Dear PVS-Studio, please check it.
+ * PVS-Studio Static Code Analyzer for C, C++ and C#: http://www.viva64.com
+ */
+
 #include <config.h>
 
 #include <sys/types.h>
-#include <sys/param.h>
 #include <sys/stat.h>
 
 #include <stdio.h>
-#ifdef STDC_HEADERS
-# include <stdlib.h>
-# include <stddef.h>
-#else
-# ifdef HAVE_STDLIB_H
-#  include <stdlib.h>
-# endif
-#endif /* STDC_HEADERS */
+#include <stdlib.h>
 #ifdef HAVE_STDBOOL_H
 # include <stdbool.h>
 #else
 # include "compat/stdbool.h"
 #endif /* HAVE_STDBOOL_H */
 #ifdef HAVE_STRING_H
-# if defined(HAVE_MEMORY_H) && !defined(STDC_HEADERS)
-#  include <memory.h>
-# endif
 # include <string.h>
 #endif /* HAVE_STRING_H */
 #ifdef HAVE_STRINGS_H
 # include <strings.h>
 #endif /* HAVE_STRINGS_H */
-#ifdef HAVE_UNISTD_H
-# include <unistd.h>
-#endif /* HAVE_UNISTD_H */
-#ifdef HAVE_DLOPEN
-# include <dlfcn.h>
-#else
-# include "compat/dlfcn.h"
-#endif
+#include <unistd.h>
 #include <ctype.h>
 #include <errno.h>
 #include <fcntl.h>
@@ -58,12 +45,10 @@
 #include <grp.h>
 #include <pwd.h>
 
+#include "sudo_compat.h"
+#include "sudo_dso.h"
 #include "sudo_plugin.h"
-#include "missing.h"
-
-#ifndef RTLD_DEFAULT
-# define RTLD_DEFAULT	NULL
-#endif
+#include "sudo_util.h"
 
 /*
  * Sudoers group plugin that does group name-based lookups using the system
@@ -89,16 +74,16 @@ sysgroup_init(int version, sudo_printf_t sudo_printf, char *const argv[])
 
     sudo_log = sudo_printf;
 
-    if (GROUP_API_VERSION_GET_MAJOR(version) != GROUP_API_VERSION_MAJOR) {
+    if (SUDO_API_VERSION_GET_MAJOR(version) != GROUP_API_VERSION_MAJOR) {
 	sudo_log(SUDO_CONV_ERROR_MSG,
 	    "sysgroup_group: incompatible major version %d, expected %d\n",
-	    GROUP_API_VERSION_GET_MAJOR(version),
+	    SUDO_API_VERSION_GET_MAJOR(version),
 	    GROUP_API_VERSION_MAJOR);
 	return -1;
     }
 
     /* Share group cache with sudo if possible. */
-    handle = dlsym(RTLD_DEFAULT, "sudo_getgrnam");
+    handle = sudo_dso_findsym(SUDO_DSO_DEFAULT, "sudo_getgrnam");
     if (handle != NULL) {
 	sysgroup_getgrnam = (sysgroup_getgrnam_t)handle;
     } else {
@@ -106,7 +91,7 @@ sysgroup_init(int version, sudo_printf_t sudo_printf, char *const argv[])
 	need_setent = true;
     }
 
-    handle = dlsym(RTLD_DEFAULT, "sudo_getgrgid");
+    handle = sudo_dso_findsym(SUDO_DSO_DEFAULT, "sudo_getgrgid");
     if (handle != NULL) {
 	sysgroup_getgrgid = (sysgroup_getgrgid_t)handle;
     } else {
@@ -114,7 +99,7 @@ sysgroup_init(int version, sudo_printf_t sudo_printf, char *const argv[])
 	need_setent = true;
     }
 
-    handle = dlsym(RTLD_DEFAULT, "sudo_gr_delref");
+    handle = sudo_dso_findsym(SUDO_DSO_DEFAULT, "sudo_gr_delref");
     if (handle != NULL)
 	sysgroup_gr_delref = (sysgroup_gr_delref_t)handle;
 
@@ -137,23 +122,24 @@ sysgroup_cleanup(void)
 static int
 sysgroup_query(const char *user, const char *group, const struct passwd *pwd)
 {
-    char **member, *ep = '\0';
+    char **member;
     struct group *grp;
 
     grp = sysgroup_getgrnam(group);
     if (grp == NULL && group[0] == '#' && group[1] != '\0') {
-	long lval = strtol(group + 1, &ep, 10);
-	if (*ep == '\0') {
-	    if ((lval != LONG_MAX && lval != LONG_MIN) || errno != ERANGE)
-		grp = sysgroup_getgrgid((gid_t)lval);
-	}
+	const char *errstr;
+	gid_t gid = sudo_strtoid(group + 1, NULL, NULL, &errstr);
+	if (errstr == NULL)
+	    grp = sysgroup_getgrgid(gid);
     }
     if (grp != NULL) {
-	for (member = grp->gr_mem; *member != NULL; member++) {
-	    if (strcasecmp(user, *member) == 0) {
-		if (sysgroup_gr_delref)
-		    sysgroup_gr_delref(grp);
-		return true;
+	if (grp->gr_mem != NULL) {
+	    for (member = grp->gr_mem; *member != NULL; member++) {
+		if (strcasecmp(user, *member) == 0) {
+		    if (sysgroup_gr_delref)
+			sysgroup_gr_delref(grp);
+		    return true;
+		}
 	    }
 	}
 	if (sysgroup_gr_delref)
@@ -163,7 +149,7 @@ sysgroup_query(const char *user, const char *group, const struct passwd *pwd)
     return false;
 }
 
-struct sudoers_group_plugin group_plugin = {
+__dso_public struct sudoers_group_plugin group_plugin = {
     GROUP_API_VERSION,
     sysgroup_init,
     sysgroup_cleanup,
